// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KOSMINDOORMAP_VERSION_H
#define KOSMINDOORMAP_VERSION_H

#define KOSMINDOORMAP_VERSION_STRING "25.12.3"
#define KOSMINDOORMAP_VERSION_MAJOR 25
#define KOSMINDOORMAP_VERSION_MINOR 12
#define KOSMINDOORMAP_VERSION_PATCH 3
#define KOSMINDOORMAP_VERSION ((25<<16)|(12<<8)|(3))

#endif
